//  Data Structures with Java by John R. Hubbard
//  Copyright McGraw-Hill, 2001
//  Problem 14.3 on page 291
//  An ordered concordance without common words

import java.io.*;
import java.util.*;

public class Pr1403
{ private static Map concordance;
  private static Set words;
  private static File file;
  private static FileReader reader;
  private static BufferedReader in;
      
  public static void main(String[] args)
  { loadSet("CommonWords.txt");
    dumpSet();
    loadMap("Shakespeare.txt");
    dumpMap();
  }
  
  private static void loadSet(String filename)
  { try
    { words = new HashSet();
      file = new File(filename);
      reader = new FileReader(file);
      in = new BufferedReader(reader);
      String line=null;
      StringTokenizer parser=null;
      while ((line=in.readLine()) != null)
      { parser = new StringTokenizer(line,",.;:()-!?' ");
        String word = parser.nextToken().toUpperCase();
        words.add(word);
      }
    }
    catch(Exception e) { System.out.println(e); }
  }

  private static void dumpSet()
  { for (Iterator it=words.iterator(); it.hasNext(); )
      System.out.println(it.next());
  }
  
  private static void loadMap(String filename)
  { try
    { concordance = new TreeMap();
      file = new File(filename);
      reader = new FileReader(file);
      in = new BufferedReader(reader);
      String line=null;
      StringTokenizer parser=null;
      int lineNumber=0;
      while ((line=in.readLine()) != null)
      { ++lineNumber;
        parser = new StringTokenizer(line,",.;:()-!?' ");
        while (parser.hasMoreTokens())
        { String word = parser.nextToken().toUpperCase();
          if (!words.contains(word))
          { String listing = (String)concordance.get(word);
            if (listing==null) listing = "" + lineNumber;
            else listing += ", " + lineNumber;
            concordance.put(word,listing);
          }
        }
      }
    }
    catch(Exception e) { System.out.println(e); }
  }

  private static void dumpMap()
  { Set set = concordance.entrySet();
    for (Iterator it=set.iterator(); it.hasNext(); )
      System.out.println(it.next());
  }
}
